@@ -1,5 +1,5 @@
 /*
- *   $Id: process.c,v 1.26 2011/04/04 14:24:58 reubenhwk Exp $
+ *   $Id: process.c,v 1.29 2011/08/19 16:37:05 reubenhwk Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
@@ -132,12 +132,12 @@ process(struct Interface *ifacel, unsigned char *msg, int len,
 
 	if (icmph->icmp6_type == ND_ROUTER_SOLICIT)
 	{
-		flog(LOG_WARNING, "received RS from %s", addr_str);
+		dlog(LOG_DEBUG, 4, "received RS from %s", addr_str);
 		process_rs(iface, msg, len, addr);
 	}
 	else if (icmph->icmp6_type == ND_ROUTER_ADVERT)
 	{
-		flog(LOG_WARNING, "received RA from %s", addr_str);
+		dlog(LOG_DEBUG, 4, "received RA from %s", addr_str);
 		process_ra(iface, msg, len, addr);
 	}
 }
@@ -194,13 +194,10 @@ process_rs(struct Interface *iface, unsigned char *msg, int len,
 	delay = MAX_RA_DELAY_TIME * rand() / (RAND_MAX +1.0);
 
 	if (iface->UnicastOnly) {
-		dlog(LOG_DEBUG, 3, "random mdelay for %s: %g seconds.", iface->Name, delay/1000.0);
-		mdelay(delay);
 		send_ra_forall(iface, &addr->sin6_addr);
 	}
 	else if ( timevaldiff(&tv, &iface->last_multicast) / 1000.0 < iface->MinDelayBetweenRAs ) {
 		/* last RA was sent only a few moments ago, don't send another immediately. */
-		dlog(LOG_DEBUG, 3, "random mdelay for %s: %g seconds.", iface->Name, delay/1000.0);
 		next = iface->MinDelayBetweenRAs - (tv.tv_sec + tv.tv_usec / 1000000.0) + (iface->last_multicast.tv_sec + iface->last_multicast.tv_usec / 1000000.0) + delay/1000.0;
 		iface->next_multicast = next_timeval(next);
 	}
@@ -281,7 +278,7 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 		char prefix_str[INET6_ADDRSTRLEN];
 		char rdnss_str[INET6_ADDRSTRLEN];
 		char suffix[256];
-		int offset, label_len;
+		unsigned int offset, label_len;
 		uint32_t preferred, valid, count;
 
 		if (len < 2)
@@ -301,8 +298,9 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 		}
 		else if (optlen > len)
 		{
-			flog(LOG_ERR, "option length greater than total"
-				" length in RA on %s from %s",
+			flog(LOG_ERR, "option length (%d) greater than total"
+				" length (%d) in RA on %s from %s",
+				optlen, len,
 				iface->Name, addr_str);
 			break;
 		}
@@ -311,6 +309,8 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 		{
 		case ND_OPT_MTU:
 			mtu = (struct nd_opt_mtu *)opt_str;
+			if (len < sizeof(*mtu))
+				return;
 
 			if (iface->AdvLinkMTU && (ntohl(mtu->nd_opt_mtu_mtu) != iface->AdvLinkMTU))
 			{
@@ -320,6 +320,8 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 			break;
 		case ND_OPT_PREFIX_INFORMATION:
 			pinfo = (struct nd_opt_prefix_info *) opt_str;
+			if (len < sizeof(*pinfo))
+				return;
 			preferred = ntohl(pinfo->nd_opt_pi_preferred_time);
 			valid = ntohl(pinfo->nd_opt_pi_valid_time);
 
@@ -374,6 +376,8 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 			break;
 		case ND_OPT_RDNSS_INFORMATION:
 			rdnssinfo = (struct nd_opt_rdnss_info_local *) opt_str;
+			if (len < sizeof(*rdnssinfo))
+				return;
 			count = rdnssinfo->nd_opt_rdnssi_len;
 
 			/* Check the RNDSS addresses received */
@@ -414,8 +418,13 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 			break;
 		case ND_OPT_DNSSL_INFORMATION:
 			dnsslinfo = (struct nd_opt_dnssl_info_local *) opt_str;
+			if (len < sizeof(*dnsslinfo))
+				return;
+
 			suffix[0] = '\0';
 			for (offset = 0; offset < (dnsslinfo->nd_opt_dnssli_len-1)*8;) {
+				if (&dnsslinfo->nd_opt_dnssli_suffixes[offset] - opt_str >= len)
+					return;
 				label_len = dnsslinfo->nd_opt_dnssli_suffixes[offset++];
 
 				if (label_len == 0) {
@@ -435,7 +444,14 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 					continue;
 				}
 
-				if ((sizeof(suffix) - strlen(suffix)) < (label_len + 2)) {
+				/*
+				 * 1) must not overflow int: label + 2, offset + label_len
+				 * 2) last byte of dnssli_suffix must not overflow opt_str + len
+				 */
+				if ((sizeof(suffix) - strlen(suffix)) < (label_len + 2) ||
+				    label_len > label_len + 2 ||
+				    &dnsslinfo->nd_opt_dnssli_suffixes[offset+label_len] - opt_str >= len ||
+				    offset + label_len < offset) {
 					flog(LOG_ERR, "oversized suffix in DNSSL option on %s from %s",
 							iface->Name, addr_str);
 					break;
@@ -443,7 +459,7 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 
 				if (suffix[0] != '\0')
 					strcat(suffix, ".");
-				strncat(suffix, &dnsslinfo->nd_opt_dnssli_suffixes[offset], label_len);
+				strncat(suffix, (char*)&dnsslinfo->nd_opt_dnssli_suffixes[offset], label_len);
 				offset += label_len;
 			}
 			break;
