@@ -463,7 +463,7 @@ print_ff(unsigned char *msg, int len, struct sockaddr_in6 *addr, int hoplimit, u
 
 				if (suffix[0] != '\0')
 					strcat(suffix, ".");
-				strncat(suffix, &dnssl_info->nd_opt_dnssli_suffixes[offset], label_len);
+				strncat(suffix, (char*)&dnssl_info->nd_opt_dnssli_suffixes[offset], label_len);
 				offset += label_len;
 			}
 
