@@ -1,5 +1,5 @@
 /*
- *   $Id: recv.c,v 1.13 2010/12/14 11:58:21 psavola Exp $
+ *   $Id: recv.c,v 1.14 2011/02/06 03:41:38 reubenhwk Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
@@ -19,7 +19,7 @@
 #include "radvd.h"
 
 int
-recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr,
+recv_rs_ra(unsigned char *msg, struct sockaddr_in6 *addr,
                  struct in6_pktinfo **pkt_info, int *hoplimit)
 {
 	struct msghdr mhdr;
